/* eslint-disable no-case-declarations */
import React from 'react';
import {
  Form,
  Input,
  Icon,
  Tooltip,
  Select,
  Radio,
  DatePicker,
  Cascader,
  InputNumber,
  Checkbox,
  Upload,
  Button,
  Divider,
} from 'antd';
import { position } from '../../../utils';
import { imgType } from '../../../utils/contants';

const CheckboxGroup = Checkbox.Group;
const RadioGroup = Radio.Group;
const { TextArea } = Input;
const { Option } = Select;
const FormItem = Form.Item;

// 常规表单有单行文本

const CommonType = ({ obj, deleteData }) => {
  let form;
  const { options = {} } = obj;
  switch (obj.type) {
    // 附件
    case 'attachment':
      form = (
        <Upload
          action={options.action}
          listType={options.upbtnstyle === 'square' ? 'picture-card' : 'text'}
          accept={
            options.upbtnstyle === 'square' ? imgType.join(',') : undefined
          }
        >
          {options.upbtnstyle === 'square' ? (
            <div>
              <i className="form form-add" />
              <div className="ant-upload-text">{options.placeholder}</div>
            </div>
          ) : (
            <Button>
              <i className="form form-export" /> {options.placeholder}
            </Button>
          )}
        </Upload>
      );
      break;
    // 单行文本
    case 'input':
      form = (
        <Input
          size={options.size || 'default'}
          placeholder={options.placeholder || ''}
        />
      );
      break;
    // 多行文本
    case 'textarea':
      form = (
        <TextArea
          size={options.size || 'default'}
          rows={options.rows || 3}
          placeholder={options.placeholder || ''}
        />
      );
      break;
    // 下拉框
    case 'select':
      form = (
        <Select
          size={options.size || 'default'}
          placeholder={options.placeholder || ''}
          getPopupContainer={() => document.querySelector('.form-container')}
        >
          {options.items && Array.isArray(options.items)
            ? options.items.map((item, idx) => (
                <Option key={idx} value={item.value}>
                  {item.text}
                </Option>
              ))
            : null}
        </Select>
      );
      break;
    // 单选按钮
    case 'radio':
      form = (
        <RadioGroup
          size={options.size || 'default'}
          getPopupContainer={() => document.querySelector('.form-container')}
        >
          {options.items && Array.isArray(options.items)
            ? options.items.map((item, idx) => (
                <Radio key={idx} value={item.value}>
                  {item.text}
                </Radio>
              ))
            : null}
        </RadioGroup>
      );
      break;
    // 多选
    case 'checkbox':
      form = (
        <CheckboxGroup size={options.size || 'default'}>
          {options.items && Array.isArray(options.items)
            ? options.items.map((item, idx) => (
                <Checkbox value={item.value} key={idx}>
                  {item.text}
                </Checkbox>
              ))
            : null}
        </CheckboxGroup>
      );
      break;
    // 日期选择框
    case 'datetime':
      form = (
        <DatePicker
          size={options.size || 'default'}
          placeholder={options.placeholder || ''}
          getCalendarContainer={() => document.querySelector('.form-container')}
        />
      );
      break;
    // 级联
    case 'cascade':
      form = (
        <Cascader
          showSearch
          size={options.size || 'default'}
          options={options.items || []}
          placeholder={options.placeholder || ''}
          getPopupContainer={() => document.querySelector('.form-container')}
        />
      );
      break;
    // 省市区
    // 根据options.level判断省市区类型
    // 数据结构地址: https://confluence.icsoc.net/pages/viewpage.action?pageId=21545602
    case 'city':
      form = (
        <Cascader
          showSearch
          notFoundContent="无匹配结果"
          popupClassName={options.level === 2 ? 'cascade-province' : ''}
          size={options.size || 'default'}
          options={position[options.level || 0]}
          placeholder={options.placeholder || ''}
          getPopupContainer={() => document.querySelector('.form-container')}
        />
      );
      break;
    // 手机
    case 'mobile':
      const selectAfter = (
        <div>
          <Icon type="phone" />
          <Icon type="mail" />
        </div>
      );
      form = (
        <Input
          addonAfter={selectAfter}
          size={options.size || 'default'}
          placeholder={options.placeholder || ''}
        />
      );
      break;
    // 电话
    case 'telephone':
      form = (
        <Input
          addonAfter={
            <div>
              <Icon type="phone" />
            </div>
          }
          size={options.size || 'default'}
          placeholder={options.placeholder || ''}
        />
      );
      break;
    // 邮箱
    case 'email':
      form = (
        <Input
          size={options.size || 'default'}
          placeholder={options.placeholder || ''}
        />
      );
      break;
    // 数字
    case 'number':
      form = (
        <InputNumber
          size={options.size || 'default'}
          placeholder={options.placeholder || undefined}
          min={options.min || 1}
          max={options.max || 10}
        />
      );
      break;
    // 联动
    case 'linkage':
      form = (
        <Select
          size={options.size || 'default'}
          placeholder={options.placeholder || undefined}
          getPopupContainer={() => document.querySelector('.form-container')}
        >
          {options.items && Array.isArray(options.items)
            ? options.items.map((item, idx) => (
                <Option key={idx} value={item.value}>
                  {item.text}
                </Option>
              ))
            : null}
        </Select>
      );
      break;
    case 'phone':
      const iconStyle = {
        cursor: 'pointer',
        color: 'rgb(107, 122, 136)',
        fontSize: '14px',
      };
      form = (
        <Input
          addonAfter={
            <span>
              <Icon type="phone" style={iconStyle} />
              <Divider type="vertical" />
              <Icon type="mail" style={iconStyle} />
            </span>
          }
          size={options.size || 'default'}
          placeholder={options.placeholder || ''}
        />
      );
      break;
    case 'association':
      form = (
        <Select
          size={options.size || 'default'}
          placeholder={options.placeholder}
          getPopupContainer={() => document.querySelector('.form-container')}
        />
      );
      break;
    default:
      form = <div />;
      break;
  }

  return (
    <div className="sort-box">
      <FormItem
        label={
          <div className="inner-box" title={options.label || ''}>
            {options.label || ''}
          </div>
        }
      >
        {form}
      </FormItem>
      <div className="icon-box">
        <Tooltip title="删除">
          <i
            className="form form-trash"
            onClick={() => {
              deleteData(obj.itemId, obj.id, obj);
            }}
          />
        </Tooltip>
      </div>
    </div>
  );
};

export default CommonType;
