import React, { Component } from 'react';
import { Input } from 'antd';
import Sortable from 'react-sortablejs';

// const IconFont = Icon.createFromIconfontCN({
//   scriptUrl: "//at.alicdn.com/t/font_641504_acqt4303ler.js"
// });

const ICON_CLASS = {
  line: 'form-fengexian',
  attachment: 'form-fujian',
  cascade: 'form-jilian',
  checkbox: 'form-duoxuan',
  city: 'form-shengshiqu',
  datetime: 'form-date1',
  email: 'form-message1',
  input: 'form-danhang',
  linkage: 'form-liandong',
  mobile: 'form-phone1',
  number: 'form-number',
  radio: 'form-danxuan',
  select: 'form-xiala',
  telephone: 'form-tel',
  textarea: 'form-danhang',
  phone: 'form-phone',
  association: 'form-xiala',
};

const LeftGroup = ({ items }) => {
  const arr = items.map(item => {
    const { options: { label = '' } = {}, type = null } = item || {};
    // 判断Icons的class
    const iconClassName = ICON_CLASS[type] || '';
    return (
      <li
        className={item.isDrag ? 'is-disable' : 'drag_item'}
        key={item.id}
        data-id={item.id}
        data={JSON.stringify(item)}
      >
        <div className="controlComponent__tag">
          <i className={`form ${iconClassName}`} />
          {label}
        </div>
      </li>
    );
  });
  return (
    <Sortable
      // See all Sortable options at https://github.com/RubaXa/Sortable#options
      options={{
        animation: 200,
        group: {
          name: 'form-left',
          pull: 'clone',
          put: false,
          revertClone: true,
        },
        filter: '.is-disable',
        sort: false,
      }}
    >
      {arr}
    </Sortable>
  );
};

class LeftShow extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      keyword: '', // 查询的字段名称
      tempList: [], // 渲染字段列表数据，此处是通过props传入的数据-需修改todo
    };

    this.handleScroll = this.handleScroll.bind(this);
  }

  componentDidMount() {
    const { list, filter, scrollTop } = this.props;
    this.setState(
      {
        tempList: list,
        keyword: filter,
      },
      () => {
        this.handleSearch();
      },
    );
    this.ele.scrollTop = scrollTop;
    this.ele.addEventListener('scroll', this.handleScroll);
    const that = this;
    try {
      that.domHeight = document.getElementsByClassName(
        'innerContent',
      )[0].scrollHeight;
    } catch (e) {}
    this.forceUpdate();
  }

  componentDidUpdate() {
    const { scrollTop } = this.props;
    this.ele.scrollTop = scrollTop;
  }

  componentWillUnmount() {
    this.ele.removeEventListener('scroll', this.handleScroll);
  }

  componentWillReceiveProps(nextProps) {
    const { list, filter } = nextProps;
    this.setState(
      {
        tempList: list,
        keyword: filter,
      },
      () => {
        this.handleSearch();
      },
    );
  }

  emitEmpty = () => {
    const { changeFilter } = this.props;
    this.searchInput.focus();
    changeFilter('');
    this.setState({ keyword: '' });
  };

  onChangeSearchInput = e => {
    const { changeFilter } = this.props;
    changeFilter(e.target.value);
    this.setState({ keyword: e.target.value });
  };

  // ENTER键-搜索字段名称
  handleSearch = () => {
    const { list } = this.props;
    const { tempList, keyword } = this.state;
    if (keyword === '') {
      this.setState({ tempList: list });
    } else {
      const filterList = tempList.filter(v => {
        const { options: { label = '' } = {} } = v;
        return label.includes(keyword);
      });
      this.setState({ tempList: filterList });
    }
  };

  handleScroll = () => {
    const { handleScroll } = this.props;
    handleScroll(this.ele.scrollTop);
  };

  render() {
    const { keyword, tempList } = this.state;
    const { btnArea = null } = this.props;
    const suffix = keyword ? (
      <i className="form form-close-fill" onClick={this.emitEmpty} />
    ) : null;
    return (
      <div className="formEditor">
        {/* 左侧布局 begin */}
        <div className="controlComponent">
          <h3 className="controlComponent__title">
            选择字段
            <span>（选择拖拽字段到右侧指定位置）</span>
          </h3>

          <div className="controlComponent__container">
            <div style={{ margin: '0 10px 10px' }}>
              <Input
                className="controlComponent__search"
                placeholder="请输入字段名称"
                prefix={<i className="form form-search" />}
                suffix={suffix}
                value={keyword}
                onChange={this.onChangeSearchInput}
                onPressEnter={this.handleSearch}
                ref={node => {
                  this.searchInput = node;
                }}
              />
            </div>
            <ul
              className="controlComponent__list"
              style={this.domHeight ? { height: this.domHeight - 379 } : {}}
              ref={ele => {
                this.ele = ele;
              }}
            >
              <LeftGroup key={tempList} items={tempList} />
            </ul>
          </div>
        </div>
        {/* 左侧布局 end */}
        {btnArea}
      </div>
    );
  }
}

export default LeftShow;
