import React from 'react';
import { Input, message } from 'antd';
import classnames from 'classnames';
import LeftShow from './components/leftShow';
import FormRow from './formRow'; // 表单行组件

const SortContainer = ({
  formMatrix = [],
  formData = [],
  chooseItem = () => {},
  changeItem = () => {},
  changeData = () => {},
  addItem = () => {},
  addTopRow = () => {},
  addBottomRow = () => {},
  mergeRow = () => {},
  deleteRow = () => {},
  deleteData = () => {},
}) => {
  const exLen = new Set();
  formData.forEach(item => {
    if (
      (item.type === 'attachment' && item.options.upbtnstyle === 'square') ||
      item.type === 'textarea'
    ) {
      exLen.add(item.rowKey);
    }
  });
  return (
    <div className="form-editor-container">
      {formMatrix.map((row, rowIdx) => (
        <FormRow
          addItem={addItem}
          changeItem={changeItem}
          chooseItem={chooseItem}
          data={formData.filter(item => item.rowKey === row.key)}
          length={formMatrix.length}
          rowIdx={rowIdx}
          key={rowIdx}
          rowKey={row.key}
          addTopRow={addTopRow}
          addBottomRow={addBottomRow}
          deleteRow={deleteRow}
          mergeRow={mergeRow}
          deleteData={deleteData}
          columns={row.column}
          changeData={changeData}
        />
      ))}
    </div>
  );
};

class EditForm extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      formMatrix: [],
      formData: [],
      columns: 2,
      rows: 4,
      formFlag: 0, // 做移动右边项时清空整个表单
      listFlag: true, // 清空整个左边列表
      originItems: [],
      formTitle: undefined,
      filter: '', // 左侧列表筛选字段
    };

    this.scrollTop = 0; // 左边滚动条位置
    this.originItems = []; // 左边元数据
    this.currentBoxId = 0;
    this.defaultColumns = 2;
    this.defaultRows = 4;
    this.choosedItem = {};
    this.itemId = 0; // 保证添加的每一项id不同 主要针对分割线可多次拖动
    this.dragIds = []; // 已拖拽的id 用来判断是否重复，针对于普通字段与联动字段
    this.domHeight = 0;
    this.timer = null;
  }

  componentDidMount() {
    const {
      columns = this.defaultColumns,
      rows = this.defaultRows,
      list = [],
      totalData = {},
    } = this.props;
    // 计算itemid
    if (totalData.formData && totalData.formData.length) {
      this.itemId = getMax(totalData.formData, 'itemId') + 1;
      const idArr = totalData.formData.map(item => item.id);
      list.forEach(data => {
        if (idArr.includes(data.id) && data.type !== 'line') {
          data.isDrag = true;
        }
      });
      this.dragIds = [];
      totalData.formData.forEach(item => {
        if (!this.dragIds.includes(item.id)) {
          this.dragIds.push(item.id);
        }
        if (item.type === 'linkage') {
          const { options: { items = [] } = {} } = item;
          if (Array.isArray(items)) {
            items.forEach(i => {
              if (i.optionLinks && i.optionLinks.length) {
                i.optionLinks.forEach(option => {
                  if (!this.dragIds.includes(option.id)) {
                    this.dragIds.push(option.id);
                  }
                });
              }
            });
          }
        }
      });
    }
    const formList = [
      {
        id: 'line1',
        name: '分割线',
        type: 'line',
        drags: true,
        options: {
          label: '分割线',
          defvalue: '分割线',
          placeholder: '请输入分割线名称',
        },
      },
    ].concat(list);
    const formMatrix = totalData.formMatrix || matrix(rows, columns);
    this.setState(
      {
        formMatrix,
        formData: totalData.formData || [],
        columns,
        rows,
        formTitle: totalData.formTitle || undefined,
        originItems: formList,
        listFlag: false,
      },
      () => {
        this.setState({
          listFlag: true,
        });
      },
    );
  }

  componentWillReceiveProps(nextProps) {
    const {
      columns = this.defaultColumns,
      rows = this.defaultRows,
      list = [],
      totalData = {},
    } = nextProps;
    // 计算itemid
    if (totalData.formData && totalData.formData.length) {
      this.itemId = getMax(totalData.formData, 'itemId') + 1;
      const { formData } = this.state;
      let idArr = [];
      if (formData && formData.length) {
        idArr = formData.map(item => item.id);
      } else {
        idArr = totalData.formData.map(item => item.id);
      }
      list.forEach(data => {
        if (idArr.includes(data.id) && data.type !== 'line') {
          data.isDrag = true;
        }
      });
      this.dragIds = [];
      totalData.formData.forEach(item => {
        if (!this.dragIds.includes(item.id)) {
          this.dragIds.push(item.id);
        }
        if (item.type === 'linkage') {
          const { options: { items = [] } = {} } = item;
          items.forEach(i => {
            if (i.optionLinks && i.optionLinks.length) {
              i.optionLinks.forEach(option => {
                if (!this.dragIds.includes(option.id)) {
                  this.dragIds.push(option.id);
                }
              });
            }
          });
        }
      });
    }
    const formList = [
      {
        id: 'line1',
        name: '分割线',
        type: 'line',
        drags: true,
        options: {
          label: '分割线',
          defvalue: '分割线',
          placeholder: '请输入分割线名称',
        },
      },
    ].concat(list);
    // const formMatrix = totalData.formMatrix || matrix(rows, columns);
    const formMatrix = totalData.formMatrix || this.state.formMatrix;
    const dom = document.getElementsByClassName('innerContent')[0];
    if (dom) {
      this.domHeight = dom.scrollTop;
    }
    this.setState(
      {
        formMatrix,
        columns,
        rows,
        originItems: formList,
        listFlag: false,
      },
      () => {
        this.setState({
          listFlag: true,
        });
        this.timer = setTimeout(() => {
          dom.scrollTop = this.domHeight;
        }, 10);
      },
    );
  }

  componentWillUnmount() {
    window.clearTimeout(this.timer);
  }

  // shouldComponentUpdate(nextProps, nextState) {
  //   console.log(this.props.list);
  //   console.log(nextProps.list);
  //   if (
  //     this.props.list &&
  //     nextProps.list &&
  //     nextProps.list.length &&
  //     this.props.list.length &&
  //     JSON.stringify(this.props.list) === JSON.stringify(nextProps.list)
  //   ) {
  //     return false;
  //   }
  //   return true;
  // }

  resetLeftGroup = () => {
    this.setState(
      {
        listFlag: false,
      },
      () => {
        this.setState({
          listFlag: true,
        });
      },
    );
  };

  addItem = item => {
    const { originItems, formData, formMatrix, listFlag } = this.state;
    if (
      formData.filter(
        form =>
          form.rowKey === item.rowKey && form.columnKey === item.columnKey,
      ).length
    ) {
      message.error('此单元格已有数据');
      this.resetLeftGroup();
      return;
    }
    if (
      item.type === 'line' &&
      formData.filter(form => form.rowKey === item.rowKey).length
    ) {
      message.error('此行已有数据,不能添加在此行添加分割线');
      this.resetLeftGroup();
      return;
    }
    // 拖拽时判断id是否重复-针对普通字段与联动字段
    if (item.type !== 'line') {
      if (this.dragIds.includes(item.id)) {
        message.error('联动字段选项和其他类型重复创建');
        this.resetLeftGroup();
        return;
      }
      const ids = [item.id];
      let idFlag = false;
      if (item.type === 'linkage') {
        const { options: { items = [] } = {} } = item;
        items.forEach(i => {
          if (i.optionLinks && i.optionLinks.length) {
            i.optionLinks.forEach(option => {
              if (this.dragIds.includes(option.id)) {
                idFlag = true;
              }
              if (!ids.includes(option.id)) {
                ids.push(option.id);
              }
            });
          }
        });
      }
      if (idFlag) {
        message.error('联动字段选项和其他类型重复创建');
        this.resetLeftGroup();
        return;
      }
      this.dragIds = this.dragIds.concat(ids);
    }
    originItems.forEach(origin => {
      if (item.id === origin.id) {
        this.itemId++;
        item.itemId = this.itemId;
        origin.isDrag = true;
        // 分割线需要自动占据一整行且可以一直拖拽
        if (item.drags) {
          origin.isDrag = false;
          item.columnKey = 1;
          formMatrix.forEach(row => {
            if (row.key === item.rowKey) {
              row.column = [1];
            }
          });
        }
      }
    });

    formData.push(item);
    this.setState(
      {
        formData,
        originItems,
        formMatrix,
        listFlag: false,
      },
      () => {
        this.setState({
          listFlag: true,
        });
      },
    );
  };

  changeItem = (row, column) => {
    const { formData, formMatrix } = this.state;
    let flag = false;
    // 检查此单元格是否有数据
    if (
      formData.filter(form => form.rowKey === row && form.columnKey === column)
        .length
    ) {
      message.error('此单元格已有数据');
    } else {
      formData.forEach(item => {
        if (
          item.rowKey === this.choosedItem.rowKey &&
          item.columnKey === this.choosedItem.columnKey
        ) {
          // 分割线需要自动占据一整行且可以一直拖拽
          if (item.type === 'line') {
            if (formData.filter(form => form.rowKey === row).length) {
              flag = true;
              message.error('此行已有数据,不能添加在此行添加分割线');
              return;
            }
            item.columnKey = 1;
            formMatrix.forEach(form => {
              if (form.key === row) {
                form.column = [1];
              }
            });
          }
          item.rowKey = row;
          item.columnKey = column;
        }
      });
    }

    this.setState({
      formFlag: this.state.formFlag + 1,
      formData,
      formMatrix,
    });
  };

  chooseItem = (rowKey, columnKey) => {
    this.choosedItem = {
      rowKey,
      columnKey,
    };
  };

  // 在表单行上方插入行
  addTopRow = idx => {
    const { formMatrix, columns } = this.state;
    const key = getMax(formMatrix, 'key') + 1;
    formMatrix.splice(idx, 0, { key, column: rangeNum(columns) });
    this.setState({
      formMatrix,
    });
  };

  // 在表单行下方插入行
  addBottomRow = idx => {
    const { formMatrix, columns } = this.state;
    const key = getMax(formMatrix, 'key') + 1;
    formMatrix.splice(idx + 1, 0, { key, column: rangeNum(columns) });
    this.setState({
      formMatrix,
    });
  };

  // 合并行
  mergeRow = (idx, rowKey) => {
    const { formMatrix, formData } = this.state;
    const formRow = formMatrix[idx];
    formRow.column = [1];
    formData.forEach(item => {
      if (item.rowKey === rowKey) {
        item.columnKey = 1;
      }
    });
    this.setState({
      formMatrix,
      formData,
    });
  };

  // 删除行
  deleteRow = idx => {
    const { formMatrix } = this.state;
    formMatrix.splice(idx, 1);
    this.setState({
      formMatrix,
    });
  };

  // 删除此表单
  deleteData = (itemId, id, obj) => {
    const { formData, originItems } = this.state;
    const ids = [id];
    if (obj.type === 'linkage') {
      const { options: { items = [] } = {} } = obj;
      items.forEach(i => {
        if (i.optionLinks && i.optionLinks.length) {
          i.optionLinks.forEach(option => {
            if (!ids.includes(option.id)) {
              ids.push(option.id);
            }
          });
        }
      });
    }

    this.dragIds = this.dragIds.filter(item => !ids.includes(item));
    originItems.forEach(origin => {
      if (id === origin.id) {
        origin.isDrag = false;
      }
    });
    this.setState(
      {
        formData: formData.filter(item => item.itemId !== itemId),
        originItems,
        listFlag: false,
      },
      () => {
        this.setState({
          listFlag: true,
        });
      },
    );
  };

  // 修改分割线
  changeData = (id, option, value) => {
    const { formData } = this.state;
    formData.forEach(item => {
      if (id === item.itemId) {
        item.options[option] = value;
      }
    });
    this.setState({
      formData,
    });
  };

  // 修改表单名称
  onChangeTitle = e => {
    this.setState({
      formTitle: e.target.value,
    });
  };

  // 保存编辑好的表单数据
  onSave = () => {
    const { formTitle, formData, formMatrix } = this.state;
    const filterData = formData.map(item => {
      if (item.type === 'line') {
        return item;
      }
      return {
        id: item.id,
        type: item.type,
        rowKey: item.rowKey,
        columnKey: item.columnKey,
        itemId: item.itemId,
      };
    });
    const obj = {
      formTitle,
      formData: filterData,
      formMatrix,
    };
    return obj;
  };

  changeFilter = value => {
    this.setState({
      filter: value,
    });
  };

  handleLeftScroll = scrollTop => {
    this.scrollTop = scrollTop;
  };

  render() {
    const {
      formMatrix,
      formData,
      formFlag,
      listFlag,
      originItems,
      formTitle,
      filter,
    } = this.state;
    const { noTitle = false, className = null, btnArea = null } = this.props;

    return (
      <div className={classnames('form-container', className)}>
        {listFlag ? (
          <LeftShow
            list={originItems}
            filter={filter}
            changeFilter={this.changeFilter}
            scrollTop={this.scrollTop}
            handleScroll={this.handleLeftScroll}
            btnArea={btnArea}
          />
        ) : null}
        <div className="form-right-container">
          {noTitle ? null : (
            <div className="form-title">
              <span>表单名称：</span>
              <Input
                value={formTitle}
                onChange={this.onChangeTitle}
                placeholder="请输入名称"
              />
            </div>
          )}
          <SortContainer
            key={formFlag}
            addTopRow={this.addTopRow}
            addBottomRow={this.addBottomRow}
            deleteRow={this.deleteRow}
            mergeRow={this.mergeRow}
            deleteData={this.deleteData}
            addItem={this.addItem}
            changeItem={this.changeItem}
            chooseItem={this.chooseItem}
            changeData={this.changeData}
            formMatrix={formMatrix}
            formData={formData}
          />
        </div>
      </div>
    );
  }
}

export default EditForm;

function getMax(arr, key) {
  let max = 0;
  arr.forEach(item => {
    if (item[key] > max) {
      max = item[key];
    }
  });
  return max;
}

function matrix(row, column) {
  const arr = [];
  for (let i = 1; i <= row; i++) {
    const obj = {
      key: i,
      column: rangeNum(column),
    };
    arr.push(obj);
  }

  return arr;
}

function rangeNum(num) {
  if (!isNumber(num) || num < 1) return [];
  const arr = [];
  for (let i = 1; i <= num; i++) {
    arr.push(i);
  }
  return arr;
}

// 判断是否为数字
function isNumber(value) {
  const patrn = /^(-)?\d+(\.\d+)?$/;
  if (patrn.exec(value) == null || value == '') {
    return false;
  }
  return true;
}
