import React from 'react';
import Sortable from 'react-sortablejs';
import { Col } from 'antd';
import FormType from './components/formType';

const SortBox = ({
  items = [],
  addItem = () => {},
  changeItem = () => {},
  chooseItem = () => {},
  deleteData = () => {},
  changeData = () => {},
}) => (
  <Sortable
    key={items}
    options={{
      group: {
        name: 'form-right',
        put: ['form-right', 'form-left'],
      },
      onChoose: evt => {
        chooseItem();
      },
      onAdd: a => {
        const { item } = a;

        // 获取绑定在左边拖拽元素的数据
        const data = JSON.parse(item.getAttribute('data'));
        // 获取拖拽过来的父元素
        const parentNode = item.parentNode;
        // 删除拖拽过来的元素
        parentNode.removeChild(item);
        if (data) {
          addItem(data);
        } else {
          changeItem();
        }
      },
    }}
  >
    {items.map((i, idx) => (
      <FormType
        key={idx}
        obj={i}
        deleteData={deleteData}
        changeData={changeData}
      />
    ))}
  </Sortable>
);

class FormGrid extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      items: [],
    };
  }

  componentDidMount() {
    const { data } = this.props;
    this.setState({
      items: data,
    });
  }

  componentWillReceiveProps(nextProps) {
    this.setState({
      items: nextProps.data,
    });
  }

  addItem = data => {
    const { addItem = () => {}, rowKey, columnKey } = this.props;
    data.rowKey = rowKey;
    data.columnKey = columnKey;
    addItem(data);
  };

  chooseItem = () => {
    const { chooseItem = () => {}, rowKey, columnKey } = this.props;
    chooseItem(rowKey, columnKey);
  };

  changeItem = data => {
    const { changeItem = () => {}, rowKey, columnKey } = this.props;
    changeItem(rowKey, columnKey);
  };

  render() {
    const { items } = this.state;
    const {
      columns = [1, 2],
      deleteData = () => {},
      changeData = () => {},
    } = this.props;
    return (
      <Col span={columns && columns.length ? 24 / columns.length : 12}>
        <SortBox
          items={items}
          addItem={this.addItem}
          chooseItem={this.chooseItem}
          changeItem={this.changeItem}
          deleteData={deleteData}
          changeData={changeData}
        />
      </Col>
    );
  }
}

export default FormGrid;
