import React from 'react';
import { Row, Icon } from 'antd';
import FormGrid from './formGrid'; // 表单单元格组件

class FormRow extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      data: [],
      length: 1,
    };
  }

  componentDidMount() {
    const { data = [], length } = this.props;
    this.setState({
      data,
      length,
    });
  }

  componentWillReceiveProps(nextProps) {
    this.setState({
      data: nextProps.data,
      length: nextProps.length,
    });
  }

  addTopRow = () => {
    const { rowIdx, addTopRow = () => {} } = this.props;
    addTopRow(rowIdx);
  };

  addBottomRow = () => {
    const { rowIdx, addBottomRow = () => {} } = this.props;
    addBottomRow(rowIdx);
  };

  mergeRow = () => {
    const { rowIdx, rowKey, mergeRow = () => {} } = this.props;
    mergeRow(rowIdx, rowKey);
  };

  deleteRow = () => {
    const { rowIdx, deleteRow = () => {} } = this.props;
    deleteRow(rowIdx);
  };

  render() {
    const {
      columns,
      addItem = () => {},
      rowKey,
      changeItem = () => {},
      chooseItem = () => {},
      deleteData = () => {},
      changeData = () => {},
    } = this.props;

    const { data = [], length = 1 } = this.state;
    // 给多行文本单独添加class
    let className = '';
    if (data.filter(item => item.type === 'textarea').length) {
      className = 'form-textarea';
    }
    if (data.filter(item => item.type === 'line').length) {
      className = 'form-line';
    }
    return (
      <Row className={className}>
        {columns.map((column, columnIdx) => (
          <FormGrid
            addItem={addItem}
            changeItem={changeItem}
            chooseItem={chooseItem}
            deleteData={deleteData}
            changeData={changeData}
            columns={columns}
            rowKey={rowKey}
            columnKey={column}
            key={columnIdx}
            data={data.filter(item => item.columnKey === column)}
          />
        ))}
        <div className="form-row-tip">
          <Icon type="bars" />
          <div className="form-row-tip-content">
            <div onClick={this.addTopRow}>上方插入行</div>
            <div onClick={this.addBottomRow}>下方插入行</div>
            {(data && data.length > 1) || columns.length <= 1 ? null : (
              <div onClick={this.mergeRow}>合并行</div>
            )}
            {(data && data.length > 0) || length === 1 ? null : (
              <div onClick={this.deleteRow}>删除行</div>
            )}
          </div>
        </div>
      </Row>
    );
  }
}

export default FormRow;
