import React from 'react';
import { Col, Row } from 'antd';

class FormGrid extends React.Component {
  constructor(props) {
    super(props);
    this.state = {};
  }

  render() {
    const {
      columns = [1, 2],
      data = [],
      formValue = {},
      smallScreen = false,
      defaultValue,
    } = this.props;

    return (
      <Col
        span={
          columns && columns.length
            ? 24 / (smallScreen ? 1 : columns.length)
            : 12
        }
        style={{ border: data.length === 0 ? 'none' : undefined }}
      >
        {data.map((i, idx) => {
          const {
            type,
            options: { label = '', defvalue = '', unit } = {},
            name,
          } = i;
          if (type === 'attachment') {
            const formItem = formValue[name];
            if (!formItem) return null;
            const { value = [] } = formItem;
            return (
              <Row key={idx}>
                <Col span={7}>{label}</Col>
                <Col span={1}>:</Col>
                <Col span={16}>
                  {Array.isArray(value) && value.length
                    ? value.map((a, index) => (
                        <Row key={index}>
                          <a href={a.url || ''} target="_blank">
                            {a.name || ''}
                          </a>
                        </Row>
                      ))
                    : defaultValue}
                </Col>
              </Row>
            );
          }
          if (type === 'line') {
            return (
              <Row key={idx} className="result-line">
                {defvalue}
              </Row>
            );
          }
          if (type === 'textarea') {
            const formItem = formValue[name];
            let value = defaultValue;
            if (formItem) {
              value = formItem.value;
              value = value.replace(/\n/g, '<br/>');
            }
            return (
              <React.Fragment key={idx}>
                <Row>
                  <Col span={7}>
                    <p>{label}</p>
                  </Col>
                  <Col span={1}>:</Col>
                  <Col
                    span={16}
                    dangerouslySetInnerHTML={{
                      __html: value === '' ? defaultValue : value,
                    }}
                  />
                </Row>
              </React.Fragment>
            );
          }
          const formItem = formValue[name];
          const unitText = unit ? `(${unit})` : '';
          return (
            <React.Fragment key={idx}>
              <Row>
                <Col span={7}>
                  <p>{label}</p>
                </Col>
                <Col span={1}>:</Col>
                <Col span={16}>
                  {formItem && formItem.value && formItem.value.length
                    ? formItem.value + unitText
                    : defaultValue}
                </Col>
              </Row>
            </React.Fragment>
          );
        })}
      </Col>
    );
  }

  // return (
  //   <Col
  //     span={
  //       columns && columns.length
  //         ? 24 / (smallScreen ? 1 : columns.length)
  //         : 12
  //     }
  //   >
  //     {data.map((i, idx) => {
  //       const { type, options: { label = "" } = {}, name } = i;

  //       return (
  //         <React.Fragment key={idx}>
  //           <span>{label}</span>:<span>{label}</span>
  //         </React.Fragment>
  //       );
  //     })}
  //   </Col>
  // );
}

export default FormGrid;
