import React from 'react';
import { Form } from 'antd';
import classnames from 'classnames';
import FormRow from './formRow'; // 表单行组件

class ResultForm extends React.Component {
  constructor(props) {
    super(props);
    this.state = { formValue: {}, formData: [], formMatrix: [] };
  }

  componentDidMount() {
    const { formValue = {}, formData = [], formMatrix = [] } = this.props;
    const newFormData = deepClone(formData);
    this.setState({
      formValue,
      formData: newFormData,
      formMatrix,
    });
  }

  componentWillReceiveProps(nextProps) {
    const { formValue, formData = [], formMatrix = [] } = nextProps;
    const newFormData = deepClone(formData);
    this.setState({
      formValue,
      formData: newFormData,
      formMatrix,
    });
  }

  render() {
    const {
      smallScreen = false,
      styleType = 'default',
      defaultValue = '',
      classname = null,
    } = this.props;
    let { formData, formMatrix, formValue } = this.state;
    // let { formData, formMatrix } = formInfo;
    // let { formData, formMatrix } = this.form_info;
    formData = flat(formData, formValue);
    // console.log(formData);
    return (
      <div
        className={classnames('form-result-container', styleType, classname)}
      >
        {formMatrix.map((row, rowIdx) => (
          <FormRow
            formValue={formValue}
            data={formData.filter(item => item.rowKey === row.key)}
            rowIdx={rowIdx}
            key={rowIdx}
            rowKey={row.key}
            columns={row.column}
            smallScreen={smallScreen}
            defaultValue={defaultValue}
          />
        ))}
      </div>
    );
  }
}

export default Form.create()(ResultForm);

// 将联动字段下的字段转换为平级的字段
function flat(data, formValue = {}) {
  const ids = data.map(item => item.id);
  data.forEach(obj => {
    if (obj.type === 'linkage') {
      const { rowKey, columnKey, name, options: { items = [] } = {} } = obj;
      const formitem = formValue[name] || {};
      const { id = '' } = formitem;
      items.forEach(item => {
        const { optionLinks = [], value } = item;
        if (id === value) {
          optionLinks.forEach(option => {
            option.rowKey = rowKey;
            option.columnKey = columnKey;
            if (!ids.includes(option.id)) {
              data.push(option);
              ids.push(option.id);
            }
          });
        }
      });
    }
  });
  return data;
}

// 深拷贝
// function deepClone(initalObj, finalObj) {
//   var obj = finalObj || {};
//   for (var i in initalObj) {
//     if (typeof initalObj[i] === "object") {
//       obj[i] = initalObj[i].constructor === Array ? [] : {};
//       arguments.callee(initalObj[i], obj[i]);
//     } else {
//       obj[i] = initalObj[i];
//     }
//   }
//   return obj;
// }
function deepClone(obj) {
  const objClone = Array.isArray(obj) ? [] : {};
  if (obj && typeof obj === 'object') {
    for (const key in obj) {
      if (obj.hasOwnProperty(key)) {
        // 判断ojb子元素是否为对象，如果是，递归复制
        if (obj[key] && typeof obj[key] === 'object') {
          objClone[key] = deepClone(obj[key]);
        } else {
          // 如果不是，简单复制
          objClone[key] = obj[key];
        }
      }
    }
  }
  return objClone;
}
