/* eslint-disable no-param-reassign */
import React from 'react';
import { Upload, Button, Icon, Form } from 'antd';
import { getCookie, efc, getMixProps } from '../../../utils';
import { imgType } from '../../../utils/contants';

const { Item: FormItem } = Form;

/**
 * 抽离attachment类型，兼容CRM(单附件)和工单(多附件)差异
 * 抽离后额外优化了原来所在的commonType组件，
 * 未抽离前，除了attachment其他common表单的componentWillReceiveProps也会执行逻辑
 */
class Attachment extends React.Component {
  constructor(props) {
    super(props);
    // 已上传附件本地暂存
    this.storeFileList = [];
    this.token = { 'access-token': getCookie('ekt_access_token') };
    // 是否图片类型上传
    this.isSquare = (props.obj.options || {}).upbtnstyle === 'square';
    // 最大上传数
    this.maxFileNum = (props.obj.options || {}).maxFileNum;
  }

  // 处理附件校验不通过后表单附件重新赋值问题
  componentWillReceiveProps(nextProps) {
    const { obj, form } = nextProps;
    const { setFieldsValue, getFieldValue } = form;
    const fileList = getFieldValue(obj.name);
    if (fileList && Array.isArray(fileList)) {
      if (JSON.stringify(this.storeFileList) !== JSON.stringify(fileList)) {
        let list = fileList.filter(item => !item.mark);
        // 根据maxFileNum限制上传文件数量
        if (typeof this.maxFileNum === 'number') {
          list = list.slice(-this.maxFileNum);
        }
        this.storeFileList = list;
        setFieldsValue({
          [obj.name]: list,
        });
      }
    }
  }

  normFile = e => {
    const files = Array.isArray(e) ? e : e && (e.fileList || []);
    return files;
  };

  getUploadChildComponent = () => {
    const { obj: { options: { placeholder } = {} } } = this.props;
    if (!this.isSquare) {
      return (
        <Button>
          <Icon type="upload" /> {placeholder}
        </Button>
      );
    }
    const isEqual =
      typeof this.maxFileNum === 'number' &&
      this.maxFileNum === this.storeFileList.length;
    // 图片附件
    if (!isEqual) {
      return (
        <div>
          <Icon type="plus" />
          <div className="ant-upload-text">{placeholder}</div>
        </div>
      );
    }
    // 图片数量等于最大上传数量时，隐藏按钮
    return null;
  };

  render() {
    const {
      form,
      form: { getFieldDecorator } = {},
      obj = {},
      customProps,
      smallScreen = false,
      formItemLayout: layout = {},
    } = this.props;
    const props = getMixProps(customProps, obj);
    const option = obj.options || {};
    const { readonly = false } = option;

    if (!obj.id) return <div />;

    const { rules } = option;
    let newRules = [];
    if (rules && Array.isArray(rules) && rules.length) {
      newRules = rules.map(item => {
        if (item && item.required && !item.message) {
          item.message = `${option.label}不能为空`;
        }
        return item;
      });
    }
    newRules = newRules.filter(item => item);
    const fieldProps = {
      valuePropName: 'fileList',
      // initialValue: option.defvalue,
      rules: newRules,
      getValueFromEvent: this.normFile,
    };
    const formItemLayout = smallScreen ? layout : {};

    const formData = (
      <Upload
        headers={this.token}
        size={option.size || 'default'}
        action={option.action}
        listType={this.isSquare ? 'picture-card' : 'text'}
        disabled={readonly}
        accept={this.isSquare ? imgType.join(',') : undefined}
        {...props}
        beforeUpload={(file, fileList) => {
          const { beforeUpload = () => {} } = props;
          return beforeUpload(file, fileList, obj, form);
        }}
        onChange={fileProps => {
          const { file, fileList = {} } = fileProps;
          const { response = {} } = file;
          /**
           * 兼容工单和CRM上传成功返回的数据结构差异
           * 工单：{ code: 0, data: 'https://...', message: '' }
           * CRM：{ code: 200, data: { file_name, file_path, file_size, file_url }, message: '' }
           * 工单成功返回code：0，CRM成功返回code：200
           */
          if ([0, 200].includes(response.code)) {
            const data = response.data.file_url || response.data;
            file.thumbUrl = data;
            fileList.forEach(item => {
              if (item.uid === file.uid) {
                item.thumbUrl = data;
              }
            });
          } else {
            fileList.forEach(item => {
              if (item.uid === file.uid) {
                item.status = 'error';
                item.response = response.message;
              }
            });
          }

          const { onChange = efc } = props;
          return onChange(fileProps, obj, form);
        }}
        onPreview={file => {
          const { response: { data = '' } = {}, url = '' } = file;
          // 兼容工单和CRM上传成功返回的数据结构差异
          window.open(data.file_url || data || url);
        }}
      >
        {this.getUploadChildComponent()}
      </Upload>
    );

    return (
      <FormItem
        label={
          <div className="inner-box" title={option.label || ''}>
            {option.label || ''}
          </div>
        }
        {...formItemLayout}
      >
        {getFieldDecorator(`${obj.name}`, fieldProps)(formData)}
      </FormItem>
    );
  }
}

export default Attachment;
