/* eslint-disable no-param-reassign */
import React from 'react';
import { Form, Cascader } from 'antd';
import { position, getMixProps } from '../../../utils';

const { Item: FormItem } = Form;

/**
 * 抽离city类型，兼容CRM(省市区、省市、省)和工单(省市区)差异
 * 根据options.level判断省市区类型
 * 数据结构参考: https://confluence.icsoc.net/pages/viewpage.action?pageId=21545602
 */
class City extends React.Component {
  city = e => {
    const {
      form: { setFieldsValue } = {},
      obj: { name = '' } = {},
    } = this.props;
    setFieldsValue({ [`${name}`]: e });
    return e;
  };

  render() {
    const {
      obj = {},
      form: { getFieldDecorator } = {},
      customProps,
      smallScreen = false,
      formItemLayout: layout = {},
    } = this.props;
    const props = getMixProps(customProps, obj);
    const option = obj.options || {};
    const { readonly = false } = option;

    if (!obj.id) return <div />;

    const { rules } = option;
    let newRules = [];
    if (rules && Array.isArray(rules) && rules.length) {
      newRules = rules.map(item => {
        if (item && item.required && !item.message) {
          item.message = `${option.label}不能为空`;
        }
        return item;
      });
    }
    newRules = newRules.filter(item => item);
    const initialValue =
      option.defvalue && option.defvalue.length ? option.defvalue : undefined;
    const fieldProps = {
      valuePropName: 'value',
      initialValue,
      rules: newRules,
      getValueFromEvent: this.city,
    };

    let specialInitialValue;
    if (initialValue) {
      obj.options.items.map(v => {
        if (initialValue === v.value) {
          specialInitialValue = v.text;
        }
        return v;
      });
    }
    const formItemLayout = smallScreen ? layout : {};

    const formData = (
      <Cascader
        showSearch
        notFoundContent="无匹配结果"
        popupClassName={option.level === 2 ? 'cascade-province' : ''}
        size={option.size || 'default'}
        options={position[option.level || 0]}
        placeholder={option.placeholder || ''}
        disabled={readonly}
        expandTrigger="hover"
        getPopupContainer={triggerNode => triggerNode.parentNode}
        {...props}
      />
    );

    return (
      <React.Fragment>
        <FormItem
          label={
            <div className="inner-box" title={option.label || ''}>
              {option.label || ''}
            </div>
          }
          {...formItemLayout}
        >
          {getFieldDecorator(`${obj.name}_ids`, fieldProps)(formData)}
        </FormItem>
        <FormItem style={{ display: 'none' }}>
          {getFieldDecorator(`${obj.name}`, {
            initialValue: specialInitialValue,
          })(<div />)}
        </FormItem>
      </React.Fragment>
    );
  }
}

export default City;
