/* eslint-disable no-param-reassign */
import React from 'react';
import { Form, Select } from 'antd';
import moment from 'moment';
import FormType from '../formType';
import { specialForm } from '../../../utils/contants';
import { getMixProps, efc } from '../../../utils';

const FormItem = Form.Item;
const { Option } = Select;

// 表单回显过滤单选下拉不在列表中的值
function editSingleFilterUseless(value, items) {
  if (!items || !Array.isArray(items) || !items.length) return undefined;
  let select;
  items.forEach(item => {
    if (item.value == value) {
      select = item;
    }
  });
  return select;
}

// 表单回显过滤多选不在列表中的值
function editCheckFilterUseless(valueArr, items, finalArr) {
  if (
    !items ||
    !Array.isArray(items) ||
    !items.length ||
    !valueArr ||
    !Array.isArray(valueArr) ||
    !valueArr.length
  )
    return undefined;
  items.forEach(item => {
    if (valueArr.includes(item.value)) {
      finalArr.push(item.value);
    }
  });
  return finalArr;
}

// 表单回显过滤级联不在列表中的值
function editCascadeFilterUseless(valueArr, items, finalArr) {
  if (
    !items ||
    !Array.isArray(items) ||
    !items.length ||
    !valueArr ||
    !Array.isArray(valueArr) ||
    !valueArr.length
  )
    return undefined;
  valueArr.forEach(value => {
    items.forEach(item => {
      if (item.value === value) {
        finalArr.push(value);
        valueArr = valueArr.filter(arr => arr !== value);
        if (valueArr.length) {
          return editCascadeFilterUseless(valueArr, item.children, finalArr);
        }
        return finalArr;
      }
    });
  });
}

function valueToLebal(valueArr, items, finalArr) {
  if (
    !items ||
    !Array.isArray(items) ||
    !items.length ||
    !valueArr ||
    !Array.isArray(valueArr) ||
    !valueArr.length
  )
    return undefined;
  valueArr.forEach(value => {
    items.forEach(item => {
      if (value === item.value) {
        finalArr.push(item.label);
        valueArr = valueArr.filter(arr => arr !== value);
        if (valueArr.length) {
          valueToLebal(valueArr, item.children, finalArr);
        }
      }
    });
  });
}

class Linkage extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      activeObj: undefined,
    };
  }

  componentDidMount() {
    const {
      obj,
      obj: { options = {} },
      customOptions: { noDefValue },
    } = this.props;
    this.linkageChange(obj, !noDefValue ? options.defvalue : undefined);
  }

  componentWillReceiveProps(nextProps) {
    const { obj, form: { getFieldValue } = {} } = this.props;
    const data = getFieldValue(`${obj.name}_ids`);
    this.linkageChange(obj, data, true);
  }

  componentDidUpdate() {}

  linkageChange = (obj, value, flag, cb = efc) => {
    const { form: { setFieldsValue } } = this.props;
    if (!flag) {
      const { name = '', options: { items = [] } = {} } = obj;
      let label;
      if (Array.isArray(items)) {
        items.forEach(item => {
          if (item.value === value) {
            label = item.text;
          }
        });
      }
      setFieldsValue({
        [`${obj.name}_ids`]: value,
        [`${obj.name}`]: label,
      });
    }

    // 获取data
    const { options: { items = [] } = {} } = obj;
    let data;
    if (Array.isArray(items)) {
      items.forEach(item => {
        if (item.value === value) {
          data = item.optionLinks;
        }
      });
    }
    this.setState({ activeObj: data }, () => {
      cb(data);
    });
  };

  updateLinksValue = links => {
    const subOption = {};
    const {
      storeFormData = {},
      customOptions: { phoneHidden = false },
      form: { setFieldsValue },
    } = this.props;
    if (links) {
      links.forEach(({ type, name, options }) => {
        if (storeFormData[name] || storeFormData[name] === 0) {
          switch (type) {
            case 'phone':
              subOption[name] = phoneHidden
                ? (storeFormData[name] || '')
                    .replace(/[A-Za-z]/g, '*')
                    .replace(/(.{3}).{1,4}(.{0})/, '$1****$2')
                : storeFormData[name];
              break;
            case 'datetime':
              subOption[name] = storeFormData[name]
                ? moment(storeFormData[name], options.dateFormat)
                : undefined;
              break;
            case 'select':
            case 'radio': {
              const val = storeFormData[`${name}_ids`];
              const selected = editSingleFilterUseless(val, options.items);
              subOption[`${name}_ids`] = selected ? selected.value : undefined;
              subOption[`${name}`] = selected ? selected.text : undefined;
              break;
            }
            case 'checkbox': {
              const valueArr = storeFormData[name];
              const finalArr = editCheckFilterUseless(
                valueArr,
                options.items,
                [],
              );
              subOption[name] = finalArr;
              break;
            }
            case 'city': {
              subOption[name] = storeFormData[name];
              subOption[`${name}_ids`] = storeFormData[name];
              break;
            }
            case 'cascade': {
              const valueArr = storeFormData[`${name}_ids`];
              const finalArr = [];
              editCascadeFilterUseless(valueArr, options.items, finalArr);
              subOption[`${name}_ids`] = finalArr;
              const arr = [];
              valueToLebal(finalArr, options.items, arr);
              subOption[name] = arr.length ? arr : undefined;
              break;
            }
            case 'attachment': {
              if (
                storeFormData[name] &&
                typeof storeFormData[name] === 'string'
              ) {
                const fileData = JSON.parse(storeFormData[name]);
                if (Array.isArray(fileData)) {
                  const files = [];
                  fileData.forEach(file => {
                    files.push(file.file);
                  });
                  subOption[name] = files;
                }
              }
              break;
            }
            case 'linkage':
              throw new Error('联动字段关联了联动字段!');
            default: {
              subOption[name] =
                storeFormData[name] || storeFormData[name] === 0
                  ? storeFormData[name]
                  : undefined;
              break;
            }
          }
        }
      });
      // console.log('设置 ', subOption);
      setFieldsValue(subOption);
    }
  };

  render() {
    const {
      obj,
      options,
      form: { getFieldDecorator } = {},
      form,
      customProps,
      customOptions,
      customOptions: { noDefValue },
      customRules,
      smallScreen = false,
      formItemLayout: layout = {},
      storeFormData,
    } = this.props;
    // console.log('联 ', storeFormData);
    const option = obj.options || {};
    const { readonly = false } = option;
    const { activeObj } = this.state;

    const props = getMixProps(customProps, obj);
    const formItemLayout = smallScreen ? layout : {};

    const filterProps = {
      initialValue: !noDefValue ? option.defvalue : undefined,
      rules: (option.rules || []).map(rule => {
        if (rule.required && !rule.message) {
          rule.message = `${option.label}不能为空`;
        }
        return rule;
      }),
    };
    return (
      <React.Fragment>
        <FormItem
          data-id={obj.id}
          label={
            <div className="inner-box" title={option.label || ''}>
              {option.label || ''}
            </div>
          }
          {...formItemLayout}
        >
          {getFieldDecorator(`${obj.name}_ids`, filterProps)(
            <Select
              size={option.size || 'default'}
              placeholder={option.placeholder || undefined}
              onChange={(value, a) => {
                this.linkageChange(obj, value, false, this.updateLinksValue);
              }}
              disabled={readonly}
              getPopupContainer={triggerNode => triggerNode.parentNode}
              {...props}
            >
              {option.items && Array.isArray(option.items)
                ? option.items.map((item, idx) => (
                    <Option key={idx} value={item.value}>
                      {item.text}
                    </Option>
                  ))
                : null}
            </Select>,
          )}
        </FormItem>
        <FormItem style={{ display: 'none' }}>
          {getFieldDecorator(`${obj.name}`)(<div />)}
        </FormItem>
        {/* {activeObj ? (
          <CommonType obj={activeObj} form={form} options={options} />
        ) : null} */}

        {activeObj
          ? activeObj.map(item => (
              <FormType
                key={item.name}
                obj={item}
                form={form}
                options={options}
                customProps={customProps}
                customOptions={customOptions}
                customRules={customRules}
                smallScreen={smallScreen}
                formItemLayout={formItemLayout}
                storeFormData={storeFormData}
              />
            ))
          : null}
      </React.Fragment>
    );
  }
}

export default Linkage;
