import React from 'react';
import { Row } from 'antd';
import FormGrid from './formGrid'; // 表单单元格组件

class FormRow extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      data: [],
    };
  }

  componentDidMount() {
    const { data = [] } = this.props;
    this.setState({
      data,
    });
  }

  componentWillReceiveProps(nextProps) {
    const { data } = nextProps;
    this.setState({
      data,
    });
  }

  render() {
    const {
      columns,
      rowKey,
      form,
      options = {},
      customProps,
      customRules,
      customOptions,
      smallScreen = false,
      formItemLayout = {},
      storeFormData,
    } = this.props;
    const { data = [] } = this.state;
    // 给多行文本单独添加class
    let className = '';
    if (data.filter(item => item.type === 'textarea').length) {
      className = 'form-textarea';
    }
    if (data.filter(item => item.type === 'line').length) {
      className = 'form-line';
    }
    return (
      <Row className={className}>
        {columns.map((column, columnIdx) => (
          <FormGrid
            columns={columns}
            rowKey={rowKey}
            columnKey={column}
            key={columnIdx}
            form={form}
            options={options}
            data={data.filter(item => item.columnKey === column)}
            customProps={customProps}
            customRules={customRules}
            customOptions={customOptions}
            smallScreen={smallScreen}
            formItemLayout={formItemLayout}
            storeFormData={storeFormData}
          />
        ))}
      </Row>
    );
  }
}

export default FormRow;
