import PositionCopy from './position';

// 获取省、省市数据
const getPosOption = type => {
  Reflect.preventExtensions(PositionCopy);
  switch (type) {
    // '省市'
    case 'city':
      return PositionCopy.map(v => ({
        value: v.value,
        label: v.label,
        children: v.children.map(t => ({ value: t.value, label: t.label })),
      }));
    // '省'
    case 'province':
      return PositionCopy.map(v => ({ value: v.value, label: v.label }));
    default:
      throw new Error('省市类型参数错误！');
  }
};

// 省市区 省市 省
export const position = [
  PositionCopy,
  getPosOption('city'),
  getPosOption('province'),
];

// 空函数 empty function
export const efc = () => {};

// 获取cookie方法
export function getCookie(c_name) {
  if (document.cookie.length > 0) {
    let c_start = document.cookie.indexOf(`${c_name}=`);
    if (c_start !== -1) {
      c_start = c_start + c_name.length + 1;
      let c_end = document.cookie.indexOf(';', c_start);
      if (c_end === -1) c_end = document.cookie.length;
      return unescape(document.cookie.substring(c_start, c_end));
    }
  }
  return '';
}

/**
 * 传入customProps属性，来对某个类型的表单自定义其props属性
 * 传入customOptions属性，来全局配置
 * 传入customRules属性，用来添加一些自定义的form校验规则
 * 示例如下：
 *
 * <ShowForm
 *   customProps={[
 *     {
 *       type: 'attachment',
 *       props: {
 *         beforeUpload: beforeCommon,
 *         ...
 *       }
 *       // 使用nameFitler对当前attachment类型下的某些具有固定name值的表单
 *       // 进行进一步的定制化的props
 *       nameFilter: {
 *         name_1: {
 *           beforeUpload: beforeName_1, // 与当前类型定义的props重名时，自动覆盖
 *           headers: 'token_123abc',
 *           useCommon: true | false, // 默认使用当前类型的统一props，定义为false时不使用
 *         },
 *         ...
 *       },
 *     },
 *     ...
 *    ]}
 *
 *    customOptions={{
 *      noDefValue: true | false, // 控制Association编辑状态下不显示默认defValue
 *      phoneHidden: true | false,
 *    }}
 *
 *    customRules={{
 *      phone: {
          pattern: /^[0-9*]*$/,
          message: '号码只能由数字与 * 组成',
        },
 *    }}
 * />
 * @param {Array} customProps 自定义属性
 * @param {Object} obj 当前表单数据
 */
export const getMixProps = (customProps, obj) => {
  const filterProps = customProps.find(item => item.type === obj.type);
  let props = {};
  if (
    filterProps &&
    filterProps.nameFilter &&
    filterProps.nameFilter[obj.name]
  ) {
    props =
      filterProps.nameFilter[obj.name].useCommon === false
        ? { ...filterProps.nameFilter[obj.name] }
        : {
            ...(filterProps.props || {}),
            ...filterProps.nameFilter[obj.name],
          };
  } else {
    props = (filterProps || {}).props || {};
  }
  return props;
};

/**
 * 获取当前表单的nameFilter属性
 * @param {Object} customProps 自定义属性
 * @param {Object} obj 当前表单数据
 * @param {Boolean} strict 不存在时是否严格返回undefined
 */
export const getNameFilterProps = (customProps, obj, strict = false) => {
  const { type, name } = obj;
  const { nameFilter = {} } =
    customProps.find(prop => prop.type === type) || {};
  return nameFilter[name] || (strict ? undefined : {});
};

/**
 * 防抖封装方法
 * @param {Function} func 需要防抖的目标方法
 * @param {Number} delay 延时(单位毫秒)
 * @param {Boolean} immediate 判断 头|尾 执行
 */
export function debounce(func, delay = 100, immediate) {
  let timeout;
  let context;
  let args;
  let stamp;

  function later() {
    const duration = Date.now() - stamp;
    if (duration < delay && duration >= 0) {
      timeout = setTimeout(later, delay - duration);
    } else {
      timeout = null;
      if (!immediate) {
        func.apply(context, args);
        context = null;
        args = null;
      }
    }
  }

  function debounced(...arg) {
    context = this;
    args = arg;
    stamp = Date.now();

    if (!timeout) {
      timeout = setTimeout(later, delay);
    }
    if (!timeout && immediate) {
      func.apply(context, args);
      context = null;
      args = null;
    }
  }

  debounced.clear = function clear() {
    if (timeout) {
      clearTimeout(timeout);
      timeout = null;
    }
  };

  return debounced;
}
